@@ -1,5 +1,5 @@
 /***********************************************************************
-    Copyright (C) 2014 Doug Torrance
+    Copyright (C) 2014-2015 Doug Torrance <dtorrance@piedmont.edu>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,7 +29,8 @@
 #include <sys/stat.h>
 #include <WINGs/WINGs.h>
 
-#define color(c) WMCreateNamedColor(screen,c,True)
+#define DEFAULT_TEXT_COLOR "light sea green"
+#define DEFAULT_BG_COLOR "black"
 
 typedef struct {
 	char *units;
@@ -37,6 +38,9 @@ typedef struct {
 	char *zip;
 	char *woeid_or_zip;
 	long int interval;
+	char *background;
+	char *text;
+	WMUserDefaults *defaults;
 } Preferences;
 
 typedef struct {
@@ -47,11 +51,16 @@ typedef struct {
 	WMButton *save;
 	WMButton *woeid;
 	WMButton *zip;
+	WMColorWell *background;
+	WMColorWell *text;
 	WMFrame *intervalFrame;
 	WMFrame *locationFrame;
 	WMFrame *units;
+	WMFrame *colors;
 	WMLabel *minutes;
 	WMLabel *woeid_zip;
+	WMLabel *backgroundLabel;
+	WMLabel *textLabel;
 	WMScreen *screen;
 	WMTextField *interval;
 	WMTextField *location;
@@ -65,6 +74,7 @@ typedef struct {
 	long int minutesLeft;
 	Preferences *prefs;
 	PreferencesWindow *prefsWindow;
+	WMFrame *frame;
 	WMLabel *icon;
 	WMLabel *text;
 	WMScreen *screen;
@@ -90,6 +100,7 @@ typedef struct {
 	RImage *icon;
 	int errorFlag;
 	char *errorText;
+	char retrieved[20];
 } Weather;
 
 Forecast *newForecast()
@@ -136,7 +147,6 @@ void freeForecast(Forecast *forecast)
 	wfree(forecast->low);
 	wfree(forecast->high);
 	wfree(forecast->text);
-
 }
 
 void freeForecastArray(ForecastArray *array)
@@ -172,16 +182,17 @@ void setError(Weather *weather, WMScreen *screen, const char *errorText)
 	strcpy(weather->errorText, errorText);
 }
 
-
 void setConditions(Weather *weather,
 		   WMScreen *screen,
 		   const char *temp,
 		   const char *text,
-		   const char *code
+		   const char *code,
+		   const char *background
 	)
 {
 	RContext *context;
 	char *filename;
+	time_t currentTime;
 
 	weather->temp = wrealloc(weather->temp, strlen(temp) + 1);
 	strcpy(weather->temp, temp);
@@ -192,10 +203,20 @@ void setConditions(Weather *weather,
 		code = wstrconcat("0", code);
 
 	context = WMScreenRContext(screen);
-	filename = wstrconcat(wstrconcat(DATADIR"/",code),".png");
-	weather->icon = RLoadImage(context,filename,0);
-	if (!weather->icon)
+	filename = wstrconcat(wstrconcat(DATADIR"/", code), ".png");
+	weather->icon = RLoadImage(context, filename, 0);
+	if (!weather->icon) {
 		setError(weather, screen, wstrconcat(filename, " not found"));
+	} else {
+		RColor color;
+		color = WMGetRColorFromColor(
+			WMCreateNamedColor(screen, background, True));
+		RCombineImageWithColor(weather->icon, &color);
+	}
+
+	currentTime = time(NULL);
+	strftime(weather->retrieved, sizeof weather->retrieved, "%l:%M %P %Z",
+		 localtime(&currentTime));
 }
 
 void setForecast(Forecast *forecast,
@@ -223,30 +244,31 @@ WMWindow *WMCreateDockapp(WMScreen *screen, const char *name, int argc, char **a
 	Window window;
 
 	display = WMScreenDisplay(screen);
-	dockapp = WMCreateWindow(screen,name);
+	dockapp = WMCreateWindow(screen, name);
 	WMRealizeWidget(dockapp);
 
 	window = WMWidgetXID(dockapp);
 
 	hints = XGetWMHints(display, window);
-	hints->flags |= WindowGroupHint|IconWindowHint|StateHint;
+	hints->flags |= WindowGroupHint | IconWindowHint | StateHint;
 	hints->window_group = window;
-    	hints->icon_window = window;
-    	hints->initial_state = WithdrawnState;
+	hints->icon_window = window;
+	hints->initial_state = WithdrawnState;
 
 	XSetWMHints(display, window, hints);
 	XFree(hints);
 
 	XSetCommand(display, window, argv, argc);
 
-	WMResizeWidget(dockapp,56,56);
+	WMResizeWidget(dockapp, 56, 56);
 	return dockapp;
 }
 
 Dockapp *newDockapp(WMScreen *screen, Preferences *prefs, int argc, char **argv)
 {
 	Dockapp *dockapp = wmalloc(sizeof(Dockapp));
-	WMFrame *frame;
+	WMColor *background;
+	WMColor *text;
 	WMWindow *window;
 
 	dockapp->screen = screen;
@@ -256,31 +278,46 @@ Dockapp *newDockapp(WMScreen *screen, Preferences *prefs, int argc, char **argv)
 
 	window = WMCreateDockapp(screen, "", argc, argv);
 
-	frame = WMCreateFrame(window);
-	WMSetFrameRelief(frame,WRSunken);
-	WMResizeWidget(frame,56,56);
-	WMSetWidgetBackgroundColor(frame,color("black"));
-	WMRealizeWidget(frame);
+	background = WMCreateNamedColor(screen, prefs->background, True);
+	if (!background) {
+		background = WMCreateNamedColor(screen, DEFAULT_BG_COLOR, True);
+		prefs->background = DEFAULT_BG_COLOR;
+	}
+	text = WMCreateNamedColor(screen, prefs->text, True);
+	if (!text) {
+		text = WMCreateNamedColor(screen, DEFAULT_TEXT_COLOR, True);
+		prefs->text = DEFAULT_TEXT_COLOR;
+	}
 
-	dockapp->text = WMCreateLabel(frame);
-	WMSetWidgetBackgroundColor(dockapp->text, color("black"));
-	WMSetLabelFont(dockapp->text, WMCreateFont(screen, "-Misc-Fixed-Medium-R-SemiCondensed--13-120-75-75-C-60-ISO10646-1"));
-	WMSetLabelTextColor(dockapp->text, color("Light sea green"));
-	WMSetLabelTextAlignment (dockapp->text, WACenter);
-	WMResizeWidget(dockapp->text,52,14);
-	WMMoveWidget(dockapp->text,2,40);
+	dockapp->frame = WMCreateFrame(window);
+	WMSetFrameRelief(dockapp->frame, WRSunken);
+	WMResizeWidget(dockapp->frame, 56, 56);
+	WMSetWidgetBackgroundColor(dockapp->frame, background);
+	WMRealizeWidget(dockapp->frame);
+
+	dockapp->text = WMCreateLabel(dockapp->frame);
+	WMSetWidgetBackgroundColor(dockapp->text, background);
+	WMSetLabelFont(
+		dockapp->text,
+		WMCreateFont(
+			screen,
+			"-Misc-Fixed-Medium-R-SemiCondensed--13-120-75-75-C-60-ISO10646-1"));
+	WMSetLabelTextColor(dockapp->text, text);
+	WMSetLabelTextAlignment(dockapp->text, WACenter);
+	WMResizeWidget(dockapp->text, 52, 14);
+	WMMoveWidget(dockapp->text, 2, 40);
 	WMRealizeWidget(dockapp->text);
 
-	dockapp->icon = WMCreateLabel(frame);
-	WMSetWidgetBackgroundColor(dockapp->icon,color("black"));
+	dockapp->icon = WMCreateLabel(dockapp->frame);
+	WMSetWidgetBackgroundColor(dockapp->icon, background);
 	WMRealizeWidget(dockapp->icon);
-	WMSetLabelImagePosition(dockapp->icon,WIPImageOnly);
-	WMResizeWidget(dockapp->icon,32,32);
-	WMMoveWidget(dockapp->icon,12,5);
+	WMSetLabelImagePosition(dockapp->icon, WIPImageOnly);
+	WMResizeWidget(dockapp->icon, 32, 32);
+	WMMoveWidget(dockapp->icon, 12, 5);
 
 	WMMapWidget(window);
-	WMMapWidget(frame);
-	WMMapSubwidgets(frame);
+	WMMapWidget(dockapp->frame);
+	WMMapSubwidgets(dockapp->frame);
 
 	return dockapp;
 }
@@ -290,32 +327,30 @@ char *getBalloonText(Weather *weather)
 	char *text;
 	int i;
 
-	text = wstrconcat("\n",weather->title);
-	text = wstrappend(text,"\n\nCurrent Conditions:\n");
+	text = wstrconcat("\n", weather->title);
+	text = wstrappend(text, "\nRetrieved: ");
+	text = wstrappend(text, weather->retrieved);
+	text = wstrappend(text, "\n\nCurrent Conditions:\n");
 	text = wstrappend(text, weather->text);
 	text = wstrappend(text, ", ");
 	text = wstrappend(text, weather->temp);
 	text = wstrappend(text, "°\n\nForecast:\n");
-	for (i = 0; i < weather->forecasts->length; i++) {
-		text = wstrappend(text,weather->forecasts->forecasts[i].day);
-		text = wstrappend(text," - ");
-		text = wstrappend(text,weather->forecasts->forecasts[i].text);
-		text = wstrappend(text,". High: ");
-		text = wstrappend(text,weather->forecasts->forecasts[i].high);
-		text = wstrappend(text,"° Low: ");
-		text = wstrappend(text,weather->forecasts->forecasts[i].low);
-		text = wstrappend(text,"°\n");
+	for (i = 0; i < weather->forecasts->length && i < 7; i++) {
+		text = wstrappend(text, weather->forecasts->forecasts[i].day);
+		text = wstrappend(text, " - ");
+		text = wstrappend(text, weather->forecasts->forecasts[i].text);
+		text = wstrappend(text, ". High: ");
+		text = wstrappend(text, weather->forecasts->forecasts[i].high);
+		text = wstrappend(text, "° Low: ");
+		text = wstrappend(text, weather->forecasts->forecasts[i].low);
+		text = wstrappend(text, "°\n");
 	}
-// as per Yahoo's Attribution Guidelines
-// (https://developer.yahoo.com/attribution/)
-	text = wstrappend(text,"\nPowered by Yahoo!\n\n");
+ /* as per Yahoo's Attribution Guidelines
+  * (https://developer.yahoo.com/attribution/) */
+	text = wstrappend(text, "\nPowered by Yahoo!\n\n");
 	return text;
 }
 
-
-
-
-
 /**************************************************
 from http://curl.haxx.se/libcurl/c/getinmemory.html
 ***************************************************/
@@ -324,7 +359,6 @@ struct MemoryStruct {
 	size_t size;
 };
 
-
 static size_t
 WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
 {
@@ -343,21 +377,26 @@ Weather *getWeather(WMScreen *screen, Preferences *prefs)
 {
 	char *url;
 	CURL *curl_handle;
-  	CURLcode res;
- 	struct MemoryStruct chunk;
+	CURLcode res;
+	struct MemoryStruct chunk;
 	Weather *weather;
 	xmlDocPtr doc;
 	xmlNodePtr cur;
 
-	url = wstrconcat("http://weather.yahooapis.com/forecastrss?u=",prefs->units);
-	if (strcmp(prefs->woeid_or_zip,"w") == 0) {
-		url = wstrappend(url,"&w=");
+	url = wstrdup("https://query.yahooapis.com/v1/public/yql?q="
+		      "select%20*%20from%20weather.forecast%20where%20woeid");
+	if (strcmp(prefs->woeid_or_zip, "w") == 0) {
+		url = wstrappend(url, "%20%3D%20");
 		url = wstrappend(url, prefs->woeid);
-	}
-	else {
-		url = wstrappend(url,"&p=");
+	} else {
+		url = wstrappend(url, "%20in%20(select%20woeid%20from%20"
+				 "geo.places(1)%20where%20text%3D%22");
 		url = wstrappend(url, prefs->zip);
+		url = wstrappend(url, "%22)");
 	}
+	url = wstrappend(url, "%20and%20u%3D'");
+	url = wstrappend(url, prefs->units);
+	url = wstrappend(url, "'&format=xml");
 
 	weather = newWeather();
 	chunk.memory = wmalloc(1);
@@ -371,7 +410,7 @@ Weather *getWeather(WMScreen *screen, Preferences *prefs)
 	curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *)&chunk);
 	curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "libcurl-agent/1.0");
 	res = curl_easy_perform(curl_handle);
-	if(res != CURLE_OK) {
+	if (res != CURLE_OK) {
 		setError(weather, screen, curl_easy_strerror(res));
 		curl_easy_cleanup(curl_handle);
 		curl_global_cleanup();
@@ -389,67 +428,76 @@ Weather *getWeather(WMScreen *screen, Preferences *prefs)
 	}
 	cur = xmlDocGetRootElement(doc);
 	if (cur == NULL) {
-		setError(weather, screen,"Empty document");
+		setError(weather, screen, "Empty document");
 		xmlFreeDoc(doc);
 		return weather;
 	}
 
-	if (xmlStrcmp(cur->name, (const xmlChar *) "rss")) {
-		setError(weather, screen,"Empty document");
-		fprintf(stderr,"document of the wrong type, root node != rss");
-		xmlFreeDoc(doc);
-		return weather;
-	}
-
-	cur = cur->children;
-	cur = cur->next;
-	cur = cur->children;
+	/* cur at "query" */
+	cur = cur->children; /* down to "results" */
+	cur = cur->children; /* down to "channel" */
+	cur = cur->children; /* down one more -- this is where we want to be */
 
 	while (cur != NULL) {
 		if ((!xmlStrcmp(cur->name, (const xmlChar *)"item"))) {
 			cur = cur->children;
 			while (cur != NULL) {
-				if ((!xmlStrcmp(cur->name, (const xmlChar *)"title"))) {
-					if ((!xmlStrcmp(xmlNodeListGetString(doc, cur->children, 1), (const xmlChar *)"City not found"))) {
-						setError(weather, screen, "Location not found\n");
-					}
+				if ((!xmlStrcmp(cur->name,
+						(const xmlChar *)"title"))) {
+					if ((!xmlStrcmp(
+						     xmlNodeListGetString(
+							     doc,
+							     cur->children, 1),
+						     (const xmlChar *)
+						     "City not found")))
+						setError(
+							weather, screen,
+							"Location not found\n");
 					setTitle(
 						weather,
-						xmlNodeListGetString(doc, cur->children, 1)
+						xmlNodeListGetString(
+							doc,
+							cur->children, 1)
 						);
 
 				}
-				if ((!xmlStrcmp(cur->name, (const xmlChar *)"condition"))) {
+				if ((!xmlStrcmp(
+					     cur->name,
+					     (const xmlChar *)"condition"))) {
 					setConditions(
 						weather,
 						screen,
-						xmlGetProp(cur,"temp"),
-						xmlGetProp(cur,"text"),
-						xmlGetProp(cur,"code")
+						xmlGetProp(cur, "temp"),
+						xmlGetProp(cur, "text"),
+						xmlGetProp(cur, "code"),
+						prefs->background
 						);
 				}
-				if ((!xmlStrcmp(cur->name, (const xmlChar *)"forecast"))) {
+				if ((!xmlStrcmp(cur->name,
+						(const xmlChar *)"forecast"))) {
 					Forecast *forecast = newForecast();
 					setForecast(
 						forecast,
-						xmlGetProp(cur,"day"),
-						xmlGetProp(cur,"low"),
-						xmlGetProp(cur,"high"),
-						xmlGetProp(cur,"text")
+						xmlGetProp(cur, "day"),
+						xmlGetProp(cur, "low"),
+						xmlGetProp(cur, "high"),
+						xmlGetProp(cur, "text")
 						);
-					appendForecast(weather->forecasts,forecast);
+					appendForecast(weather->forecasts,
+						       forecast);
 				}
 
 
 				cur = cur->next;
 			}
+		} else {
+			cur = cur->next;
 		}
-		else cur = cur->next;
 	}
 
 
 	xmlFreeDoc(doc);
-// finishing parsing xml
+/* finishing parsing xml */
 
 	wfree(chunk.memory);
 
@@ -459,39 +507,55 @@ Weather *getWeather(WMScreen *screen, Preferences *prefs)
 static void updateDockapp(void *data)
 {
 	Dockapp *dockapp = (Dockapp *)data;
-
+	WMColor *background;
+	WMColor *text;
 	WMScreen *screen = dockapp->screen;
 	Preferences *prefs = dockapp->prefs;
 	Weather *weather;
 	WMPixmap *icon;
 
-	WMSetLabelText(dockapp->text,"loading");
+	background = WMCreateNamedColor(screen, prefs->background, True);
+	text = WMCreateNamedColor(screen, prefs->text, True);
+
+	WMSetLabelText(dockapp->text, "loading");
+	WMSetWidgetBackgroundColor(dockapp->text, background);
+	WMSetLabelTextColor(dockapp->text, text);
 	WMRedisplayWidget(dockapp->text);
 
+	WMSetWidgetBackgroundColor(dockapp->frame, background);
+	WMSetWidgetBackgroundColor(dockapp->icon, background);
+
 	weather = getWeather(screen, prefs);
 
 	if (weather->errorFlag) {
 		RContext *context;
 
-		WMSetLabelText(dockapp->text,"ERROR");
+		WMSetLabelText(dockapp->text, "ERROR");
 
 		context = WMScreenRContext(screen);
-		weather->icon = RLoadImage(context,DATADIR"/na.png",0);
+		weather->icon = RLoadImage(context, DATADIR"/na.png", 0);
 		if (weather->icon) {
-			icon = WMCreatePixmapFromRImage(screen,weather->icon,0);
-			WMSetLabelImage(dockapp->icon,icon);
+			RColor color;
+
+			color = WMGetRColorFromColor(
+				WMCreateNamedColor(screen,
+						   prefs->background, True));
+			RCombineImageWithColor(weather->icon, &color);
+			icon = WMCreatePixmapFromRImage(screen,
+							weather->icon, 0);
+			WMSetLabelImage(dockapp->icon, icon);
 		}
 
-		WMSetBalloonTextForView(weather->errorText, WMWidgetView(dockapp->icon));
+		WMSetBalloonTextForView(weather->errorText,
+					WMWidgetView(dockapp->icon));
+	} else {
+		WMSetLabelText(dockapp->text, wstrconcat(weather->temp, "°"));
 
-	}
-	else {
-		WMSetLabelText(dockapp->text,wstrconcat(weather->temp,"°"));
+		icon = WMCreatePixmapFromRImage(screen, weather->icon, 0);
+		WMSetLabelImage(dockapp->icon, icon);
 
-		icon = WMCreatePixmapFromRImage(screen,weather->icon,0);
-		WMSetLabelImage(dockapp->icon,icon);
-
-		WMSetBalloonTextForView(getBalloonText(weather), WMWidgetView(dockapp->icon));
+		WMSetBalloonTextForView(getBalloonText(weather),
+					WMWidgetView(dockapp->icon));
 	}
 
 	WMRedisplayWidget(dockapp->icon);
@@ -500,63 +564,32 @@ static void updateDockapp(void *data)
 	freeWeather(weather);
 }
 
-char *WMGetPLStringForKey(WMPropList *propList, char *key)
-{
-	WMPropList *value;
-	value = WMGetFromPLDictionary(propList,WMCreatePLString(key));
-	if (value)
-		return WMGetPropListDescription(value, False);
-	else
-		return NULL;
-}
-
-char *getPreferencesDirectory()
-{
-	char *dir;
-	dir = getenv("XDG_CONFIG_DIR");
-	if (!dir)
-		dir = wstrconcat(getenv("HOME"), "/.config");
-	dir = wstrconcat(dir, "/wmforecast");
-	return dir;
-}
-
-char *getPreferencesFilename()
-{
-	char *filename;
-
-	filename = getPreferencesDirectory();
-	filename = wstrappend(filename, "/wmforecastrc");
-
-	return filename;
-}
-
 void readPreferences(Preferences *prefs)
 {
-	char *filename;
-	WMPropList *propList;
-
-	filename = getPreferencesFilename();
-	propList = WMReadPropListFromFile(filename);
-	if (propList) {
+	if (prefs->defaults) {
 		char *value;
-		value = WMGetPLStringForKey(propList, "units");
+		value = WMGetUDStringForKey(prefs->defaults, "units");
 		if (value)
 			prefs->units = value;
-		value = WMGetPLStringForKey(propList, "woeid");
+		value = WMGetUDStringForKey(prefs->defaults, "woeid");
 		if (value)
 			prefs->woeid = value;
-		value = WMGetPLStringForKey(propList, "zip");
+		value = WMGetUDStringForKey(prefs->defaults, "zip");
 		if (value)
 			prefs->zip = value;
-		value = WMGetPLStringForKey(propList, "woeid_or_zip");
+		value = WMGetUDStringForKey(prefs->defaults, "woeid_or_zip");
 		if (value)
 			prefs->woeid_or_zip = value;
-		value = WMGetPLStringForKey(propList, "interval");
+		value = WMGetUDStringForKey(prefs->defaults, "interval");
 		if (value)
 			prefs->interval = strtol(value, NULL, 10);
+		value = WMGetUDStringForKey(prefs->defaults, "background");
+		if (value)
+			prefs->background = value;
+		value = WMGetUDStringForKey(prefs->defaults, "text");
+		if (value)
+			prefs->text = value;
 	}
-
-
 }
 
 Preferences *setPreferences(int argc, char **argv)
@@ -564,89 +597,63 @@ Preferences *setPreferences(int argc, char **argv)
 	int c;
 	Preferences *prefs = wmalloc(sizeof(Preferences));
 
-	//set defaults
+	/* set defaults */
 	prefs->units = "f";
 	prefs->woeid = "2502265";
 	prefs->zip = NULL;
 	prefs->woeid_or_zip = NULL;
 	prefs->interval = 60;
-
+	prefs->background = DEFAULT_BG_COLOR;
+	prefs->text = DEFAULT_TEXT_COLOR;
+	prefs->defaults = WMGetStandardUserDefaults();
 	readPreferences(prefs);
 
-	//command line
-	while (1)
-	{
-		static struct option long_options[] =
-			{
-				{"version", no_argument, 0, 'v'},
-				{"help", no_argument, 0, 'h'},
-				{"woeid", required_argument, 0, 'w'},
-				{"units", required_argument, 0, 'u'},
-				{"zip", required_argument, 0, 'z'},
-				{"interval", required_argument, 0, 'i'},
-				{0, 0, 0, 0}
-			};
+	/* command line */
+	while (1) {
+		static struct option long_options[] = {
+			{"version", no_argument, 0, 'v'},
+			{"help", no_argument, 0, 'h'},
+			{"woeid", required_argument, 0, 'w'},
+			{"units", required_argument, 0, 'u'},
+			{"zip", required_argument, 0, 'z'},
+			{"interval", required_argument, 0, 'i'},
+			{"background", required_argument, 0, 'b'},
+			{"text", required_argument, 0, 't'},
+			{0, 0, 0, 0}
+		};
 		int option_index = 0;
 
-		c = getopt_long (argc, argv, "vhw:u:z:i:",
+		c = getopt_long(argc, argv, "vhw:u:z:i:b:t:",
 				 long_options, &option_index);
 
 		if (c == -1)
 			break;
 
-		switch (c)
-		{
+		switch (c) {
 		case 0:
 			if (long_options[option_index].flag != 0)
 				break;
-			printf ("option %s", long_options[option_index].name);
+			printf("option %s", long_options[option_index].name);
 			if (optarg)
-				printf (" with arg %s", optarg);
-			printf ("\n");
+				printf(" with arg %s", optarg);
+			printf("\n");
 			break;
 
 		case 'v':
-			printf("wmforecast %s\n\n"
-			       "Copyright (C) 2014 Doug Torrance\n"
+			printf("%s\n"
+			       "Copyright © 2014 Doug Torrance\n"
 			       "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
 			       "This is free software: you are free to change and redistribute it.\n"
-			       "There is NO WARRANTY, to the extent permitted by law.\n\n"
-			       "Written by Doug Torrance\n"
-			       , VERSION);
-			exit(0);
-
-		case 'h':
-			printf("a weather dockapp for Window Maker using the Yahoo Weather API\n\n"
-			       "Usage: wmforecast [OPTIONS]\n\n"
-			       "Options:\n"
-			       "    -v, --version        print the version number\n"
-			       "    -h, --help           print this help screen\n"
-			       "    -i, --interval <min> number of minutes between refreshes (default 60)\n"
-			       "    -u, --units <c|f>    whether to use Celsius or Fahrenheit (default f)\n"
-			       "    -w, --woeid <woeid>  Where on Earth ID (default is 2502265 for\n"
-			       "                         Sunnyvale, CA -- to find your WOEID, search\n"
-			       "                         for your city at http://weather.yahoo.com and\n"
-			       "                         look in the URL.)\n"
-			       "    -z, --zip <zip>      ZIP code or Location ID (Yahoo has deprecated this\n"
-			       "                         option and it is not guaranteed to work)\n"
-			       "Notes:\n"
-			       "* double click the icon at any time to refresh data\n\n"
-			       "* right click the icon to edit your preferences in a GUI\n\n"
-			       "* preferences may be manually configured in\n"
-			       "  XDG_CONFIG_DIR/wmforecast/wmforecastrc\n\n"
-			       "Report bugs to %s\n",
-			       PACKAGE_BUGREPORT
-				);
-
+			       "There is NO WARRANTY, to the extent permitted by law.\n"
+			       , PACKAGE_STRING);
 			exit(0);
 
 		case 'w':
 			prefs->woeid_or_zip = "w";
 			prefs->woeid = optarg;
 			break;
-
 		case 'u':
-			if ((strcmp(optarg,"f") != 0)&&(strcmp(optarg,"c") != 0)) {
+			if ((strcmp(optarg, "f") != 0) && (strcmp(optarg, "c") != 0)) {
 				printf("units must be 'f' or 'c'\n");
 				exit(0);
 			}
@@ -659,21 +666,48 @@ Preferences *setPreferences(int argc, char **argv)
 			break;
 
 		case 'i':
-			prefs->interval = strtol(optarg,NULL,10);
+			prefs->interval = strtol(optarg, NULL, 10);
+			break;
+
+		case 'b':
+			prefs->background = optarg;
+			break;
+
+		case 't':
+			prefs->text = optarg;
 			break;
 
 		case '?':
-			/* getopt_long already printed an error message. */
-			break;
+		case 'h':
+			printf("A weather dockapp for Window Maker using the Yahoo Weather API\n"
+			       "Usage: wmforecast [OPTIONS]\n"
+			       "Options:\n"
+			       "    -v, --version            print the version number\n"
+			       "    -h, --help               print this help screen\n"
+			       "    -i, --interval <min>     number of minutes between refreshes (default 60)\n"
+			       "    -u, --units <c|f>        whether to use Celsius or Fahrenheit (default f)\n"
+			       "    -w, --woeid <woeid>      Where on Earth ID (default is 2502265 for\n"
+			       "                             Sunnyvale, CA -- to find your WOEID, search\n"
+			       "                             for your city at http://weather.yahoo.com and\n"
+			       "                             look in the URL.)\n"
+			       "    -z, --zip <zip>          ZIP code or Location ID (Yahoo has deprecated this\n"
+			       "                             option and it is not guaranteed to work)\n"
+			       "    -b, --background <color> set background color\n"
+			       "    -t, --text <color>       set text color\n"
+			       "Report bugs to: %s\n"
+			       "wmforecast home page: https://github.com/d-torrance/wmforecast\n",
+			       PACKAGE_BUGREPORT
+				);
+			exit(0);
 
 		default:
-			abort ();
+
+			abort();
 		}
 	}
 
-	if (!prefs->woeid_or_zip) {
+	if (!prefs->woeid_or_zip)
 		prefs->woeid_or_zip = "w";
-	}
 
 	return prefs;
 }
@@ -687,67 +721,37 @@ static void closePreferences(WMWidget *widget, void *data)
 
 static void savePreferences(WMWidget *widget, void *data)
 {
-	char *dir;
-	char *filename;
-	char *prefsString;
 	Dockapp *d = (Dockapp *)data;
-	FILE *file;
-	int md;
-	WMPropList *prefsPL;
-	WMPropList *key;
-	WMPropList *object;
 
-	prefsPL = WMCreatePLDictionary(NULL,NULL);
+	if (WMGetButtonSelected(d->prefsWindow->celsius))
+		WMSetUDStringForKey(d->prefs->defaults, "c", "units");
+	if (WMGetButtonSelected(d->prefsWindow->fahrenheit))
+		WMSetUDStringForKey(d->prefs->defaults, "f", "units");
+	if (WMGetButtonSelected(d->prefsWindow->woeid))
+		WMSetUDStringForKey(d->prefs->defaults, "w", "woeid_or_zip");
+	if (WMGetButtonSelected(d->prefsWindow->zip))
+		WMSetUDStringForKey(d->prefs->defaults, "z", "woeid_or_zip");
+	WMSetUDStringForKey(d->prefs->defaults,
+			    WMGetTextFieldText(d->prefsWindow->woeidField),
+			    "woeid");
+	WMSetUDStringForKey(d->prefs->defaults,
+			    WMGetTextFieldText(d->prefsWindow->zipField),
+			    "zip");
+	WMSetUDStringForKey(d->prefs->defaults,
+			    WMGetTextFieldText(d->prefsWindow->interval),
+			    "interval");
+	WMSetUDStringForKey(d->prefs->defaults,
+			    WMGetColorRGBDescription(
+				    WMGetColorWellColor(
+					    d->prefsWindow->background)),
+			    "background");
+	WMSetUDStringForKey(d->prefs->defaults,
+			    WMGetColorRGBDescription(
+				    WMGetColorWellColor(
+					    d->prefsWindow->text)),
+			    "text");
 
-	key = WMCreatePLString("units");
-	if (WMGetButtonSelected(d->prefsWindow->celsius)) {
-		object = WMCreatePLString("c");
-		WMPutInPLDictionary(prefsPL, key, object);
-	}
-	if (WMGetButtonSelected(d->prefsWindow->fahrenheit)) {
-		object = WMCreatePLString("f");
-		WMPutInPLDictionary(prefsPL, key, object);
-	}
-
-	key = WMCreatePLString("woeid_or_zip");
-	if (WMGetButtonSelected(d->prefsWindow->woeid)) {
-		object = WMCreatePLString("w");
-		WMPutInPLDictionary(prefsPL, key, object);
-	}
-	if (WMGetButtonSelected(d->prefsWindow->zip)) {
-		object = WMCreatePLString("z");
-		WMPutInPLDictionary(prefsPL, key, object);
-	}
-
-	key = WMCreatePLString("woeid");
-	object = WMCreatePLString(WMGetTextFieldText(d->prefsWindow->woeidField));
-	WMPutInPLDictionary(prefsPL, key, object);
-
-	key = WMCreatePLString("zip");
-	object = WMCreatePLString(WMGetTextFieldText(d->prefsWindow->zipField));
-	WMPutInPLDictionary(prefsPL, key, object);
-
-	key = WMCreatePLString("interval");
-	object = WMCreatePLString(WMGetTextFieldText(d->prefsWindow->interval));
-	WMPutInPLDictionary(prefsPL, key, object);
-
-	// since WMWritePropListToFile only writes to files in
-	// GNUSTEP_USER_ROOT, we need to write our own version
-	prefsString = WMGetPropListDescription(prefsPL, True);
-
-	dir = getPreferencesDirectory();
-	md = mkdir(dir, 0777);
-	if (md < 0) {
-		printf("Error making directory %s: %s\n", dir,
-		       strerror(errno));
-	}
-
-	filename = getPreferencesFilename();
-	file = fopen(filename, "w");
-	if (file) {
-		fputs(prefsString, file);
-		fclose(file);
-	}
+	WMSaveUserDefaults(d->prefs->defaults);
 
 	readPreferences(d->prefs);
 	updateDockapp(d);
@@ -769,8 +773,8 @@ static void editPreferences(void *data)
 
 	d->prefsWindow->window = WMCreateWindow(d->prefsWindow->screen, "wmforecast");
 	WMSetWindowTitle(d->prefsWindow->window, "wmforecast");
-	WMSetWindowCloseAction (d->prefsWindow->window, closePreferences, d);
-	WMResizeWidget(d->prefsWindow->window, 424, 130);
+	WMSetWindowCloseAction(d->prefsWindow->window, closePreferences, d);
+	WMResizeWidget(d->prefsWindow->window, 424, 150);
 	WMRealizeWidget(d->prefsWindow->window);
 	WMMapWidget(d->prefsWindow->window);
 
@@ -784,16 +788,16 @@ static void editPreferences(void *data)
 	d->prefsWindow->celsius = WMCreateButton(d->prefsWindow->units, WBTRadio);
 	WMSetButtonText(d->prefsWindow->celsius, "Celsius");
 	WMMoveWidget(d->prefsWindow->celsius, 10, 20);
-	if (strcmp(d->prefsWindow->prefs->units,"c") == 0)
-		WMSetButtonSelected(d->prefsWindow->celsius,1);
+	if (strcmp(d->prefsWindow->prefs->units, "c") == 0)
+		WMSetButtonSelected(d->prefsWindow->celsius, 1);
 	WMRealizeWidget(d->prefsWindow->celsius);
 	WMMapWidget(d->prefsWindow->celsius);
 
 	d->prefsWindow->fahrenheit = WMCreateButton(d->prefsWindow->units, WBTRadio);
 	WMSetButtonText(d->prefsWindow->fahrenheit, "Fahrenheit");
 	WMMoveWidget(d->prefsWindow->fahrenheit, 10, 44);
-	if (strcmp(d->prefsWindow->prefs->units,"f") == 0)
-		WMSetButtonSelected(d->prefsWindow->fahrenheit,1);
+	if (strcmp(d->prefsWindow->prefs->units, "f") == 0)
+		WMSetButtonSelected(d->prefsWindow->fahrenheit, 1);
 	WMRealizeWidget(d->prefsWindow->fahrenheit);
 	WMMapWidget(d->prefsWindow->fahrenheit);
 
@@ -809,8 +813,8 @@ static void editPreferences(void *data)
 	d->prefsWindow->woeid = WMCreateButton(d->prefsWindow->locationFrame, WBTRadio);
 	WMSetButtonText(d->prefsWindow->woeid, "WOEID");
 	WMMoveWidget(d->prefsWindow->woeid, 10, 20);
-	if (strcmp(d->prefsWindow->prefs->woeid_or_zip,"w") == 0)
-		WMSetButtonSelected(d->prefsWindow->woeid,1);
+	if (strcmp(d->prefsWindow->prefs->woeid_or_zip, "w") == 0)
+		WMSetButtonSelected(d->prefsWindow->woeid, 1);
 	WMRealizeWidget(d->prefsWindow->woeid);
 	WMMapWidget(d->prefsWindow->woeid);
 
@@ -824,8 +828,8 @@ static void editPreferences(void *data)
 	d->prefsWindow->zip = WMCreateButton(d->prefsWindow->locationFrame, WBTRadio);
 	WMSetButtonText(d->prefsWindow->zip, "ZIP code");
 	WMMoveWidget(d->prefsWindow->zip, 10, 44);
-	if (strcmp(d->prefsWindow->prefs->woeid_or_zip,"z") == 0)
-		WMSetButtonSelected(d->prefsWindow->zip,1);
+	if (strcmp(d->prefsWindow->prefs->woeid_or_zip, "z") == 0)
+		WMSetButtonSelected(d->prefsWindow->zip, 1);
 	WMRealizeWidget(d->prefsWindow->zip);
 	WMMapWidget(d->prefsWindow->zip);
 
@@ -859,17 +863,52 @@ static void editPreferences(void *data)
 	WMRealizeWidget(d->prefsWindow->minutes);
 	WMMapWidget(d->prefsWindow->minutes);
 
+	d->prefsWindow->colors = WMCreateFrame(d->prefsWindow->window);
+	WMSetFrameTitle(d->prefsWindow->colors, "Colors");
+	WMResizeWidget(d->prefsWindow->colors, 265, 50);
+	WMMoveWidget(d->prefsWindow->colors, 10, 92);
+	WMRealizeWidget(d->prefsWindow->colors);
+	WMMapWidget(d->prefsWindow->colors);
+
+	d->prefsWindow->backgroundLabel = WMCreateLabel(d->prefsWindow->colors);
+	WMSetLabelText(d->prefsWindow->backgroundLabel, "Background");
+	WMSetLabelTextAlignment(d->prefsWindow->backgroundLabel, WACenter);
+	WMResizeWidget(d->prefsWindow->backgroundLabel, 70, 20);
+	WMMoveWidget(d->prefsWindow->backgroundLabel, 10, 20);
+	WMRealizeWidget(d->prefsWindow->backgroundLabel);
+	WMMapWidget(d->prefsWindow->backgroundLabel);
+
+	d->prefsWindow->background = WMCreateColorWell(d->prefsWindow->colors);
+	WMSetColorWellColor(d->prefsWindow->background, WMCreateNamedColor(d->screen, d->prefs->background, True));
+	WMMoveWidget(d->prefsWindow->background, 88, 16);
+	WMRealizeWidget(d->prefsWindow->background);
+	WMMapWidget(d->prefsWindow->background);
+
+	d->prefsWindow->textLabel = WMCreateLabel(d->prefsWindow->colors);
+	WMSetLabelText(d->prefsWindow->textLabel, "Text");
+	WMSetLabelTextAlignment(d->prefsWindow->textLabel, WACenter);
+	WMResizeWidget(d->prefsWindow->textLabel, 26, 20);
+	WMMoveWidget(d->prefsWindow->textLabel, 156, 20);
+	WMRealizeWidget(d->prefsWindow->textLabel);
+	WMMapWidget(d->prefsWindow->textLabel);
+
+	d->prefsWindow->text = WMCreateColorWell(d->prefsWindow->colors);
+	WMSetColorWellColor(d->prefsWindow->text, WMCreateNamedColor(d->screen, d->prefs->text, True));
+	WMMoveWidget(d->prefsWindow->text, 190, 16);
+	WMRealizeWidget(d->prefsWindow->text);
+	WMMapWidget(d->prefsWindow->text);
+
 	d->prefsWindow->save = WMCreateButton(d->prefsWindow->window, WBTMomentaryPush);
 	WMSetButtonText(d->prefsWindow->save, "Save");
 	WMSetButtonAction(d->prefsWindow->save, savePreferences, d);
-	WMMoveWidget(d->prefsWindow->save, 285, 100);
+	WMMoveWidget(d->prefsWindow->save, 285, 110);
 	WMRealizeWidget(d->prefsWindow->save);
 	WMMapWidget(d->prefsWindow->save);
 
 	d->prefsWindow->close = WMCreateButton(d->prefsWindow->window, WBTMomentaryPush);
 	WMSetButtonText(d->prefsWindow->close, "Close");
 	WMSetButtonAction(d->prefsWindow->close, closePreferences, d);
-	WMMoveWidget(d->prefsWindow->close, 353, 100);
+	WMMoveWidget(d->prefsWindow->close, 353, 110);
 	WMRealizeWidget(d->prefsWindow->close);
 	WMMapWidget(d->prefsWindow->close);
 }
@@ -901,9 +940,10 @@ int main(int argc, char **argv)
 	Preferences *prefs;
 	WMScreen *screen;
 
+	WMInitializeApplication("wmforecast", &argc, argv);
+
 	prefs = setPreferences(argc, argv);
 
-	WMInitializeApplication("wmforecast", &argc, argv);
 	display = XOpenDisplay("");
 
 	screen = WMCreateScreen(display, DefaultScreen(display));
@@ -913,7 +953,7 @@ int main(int argc, char **argv)
 			     refresh, dockapp);
 
 	updateDockapp(dockapp);
-	WMAddPersistentTimerHandler(60*1000, //one minute
+	WMAddPersistentTimerHandler(60*1000, /* one minute */
 				    timerHandler, dockapp);
 
 	WMScreenMainLoop(screen);
